/*
 * Copyright 2006 KonaWorks
 * Copyright 2000-2006 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.konaworks.idea.arrayindexer;

import com.intellij.codeInsight.hint.HintManager;
import com.intellij.codeInsight.hint.HintUtil;
import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.editor.EditorFactory;
import com.intellij.openapi.editor.LogicalPosition;
import com.intellij.openapi.editor.event.EditorEventMulticaster;
import com.intellij.openapi.editor.event.EditorMouseAdapter;
import com.intellij.openapi.editor.event.EditorMouseEvent;
import com.intellij.openapi.editor.event.EditorMouseMotionAdapter;
import com.intellij.openapi.editor.event.EditorMouseMotionListener;
import com.intellij.openapi.editor.event.VisibleAreaEvent;
import com.intellij.openapi.editor.event.VisibleAreaListener;
import com.intellij.openapi.editor.markup.EffectType;
import com.intellij.openapi.editor.markup.HighlighterLayer;
import com.intellij.openapi.editor.markup.HighlighterTargetArea;
import com.intellij.openapi.editor.markup.RangeHighlighter;
import com.intellij.openapi.editor.markup.TextAttributes;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.fileEditor.FileEditorManagerAdapter;
import com.intellij.openapi.fileEditor.FileEditorManagerEvent;
import com.intellij.openapi.fileEditor.FileEditorManagerListener;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.startup.StartupManager;
import com.intellij.openapi.ui.MultiLineLabelUI;
import com.intellij.openapi.util.Comparing;
import com.intellij.openapi.util.SystemInfo;
import com.intellij.psi.PsiArrayInitializerExpression;
import com.intellij.psi.PsiDocumentManager;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiExpression;
import com.intellij.psi.PsiField;
import com.intellij.psi.PsiFile;
import com.intellij.psi.impl.source.tree.injected.InjectedLanguageUtil;
import com.intellij.psi.util.PsiTreeUtil;
import com.intellij.ui.LightweightHint;
import com.intellij.util.ui.UIUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.JLabel;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;

public class ArrayIndexer implements ProjectComponent {
  private final Project myProject;
  private static TextAttributes ourReferenceAttributes;
  private RangeHighlighter myHighlighter;
  private Editor myHighlighterView;
  private Cursor myStoredCursor;
  private Info myStoredInfo;
  private TooltipProvider myTooltipProvider = null;

  private final KeyListener myEditorKeyListener = new KeyAdapter() {
    public void keyPressed(final KeyEvent e) {
      handleKey(e);
    }

    public void keyReleased(final KeyEvent e) {
      handleKey(e);
    }

    private void handleKey(final KeyEvent e) {
      int modifiers = e.getModifiers();

      if (isAltMask(modifiers)) {
        if (myTooltipProvider != null) {
          myTooltipProvider.execute();
        }
      } else {
        disposeHighlighter();
        myTooltipProvider = null;
      }
    }
  };

  private final FileEditorManagerListener myFileEditorManagerListener = new FileEditorManagerAdapter() {
    public void selectionChanged(FileEditorManagerEvent e) {
      disposeHighlighter();
      myTooltipProvider = null;
    }
  };

  private final VisibleAreaListener myVisibleAreaListener = new VisibleAreaListener() {
    public void visibleAreaChanged(VisibleAreaEvent e) {
      disposeHighlighter();
      myTooltipProvider = null;
    }
  };

  private final EditorMouseAdapter myEditorMouseAdapter = new EditorMouseAdapter() {
    public void mouseReleased(EditorMouseEvent e) {
      disposeHighlighter();
      myTooltipProvider = null;
    }
  };

  private final EditorMouseMotionListener myEditorMouseMotionListener = new EditorMouseMotionAdapter() {
    public void mouseMoved(final EditorMouseEvent e) {
      if (e.isConsumed() || myProject.isDisposed()) {
        return;
      }
      MouseEvent mouseEvent = e.getMouseEvent();
      Editor editor = e.getEditor();
      PsiFile psiFile = PsiDocumentManager.getInstance(myProject).getPsiFile(editor.getDocument());
      Point point = new Point(mouseEvent.getPoint());
      editor = InjectedLanguageUtil.getEditorForInjectedLanguage(editor, psiFile, editor.logicalPositionToOffset(editor.xyToLogicalPosition(point)));
      LogicalPosition pos = editor.xyToLogicalPosition(point);
      int offset = editor.logicalPositionToOffset(pos);
      int selStart = editor.getSelectionModel().getSelectionStart();
      int selEnd = editor.getSelectionModel().getSelectionEnd();
      int modifiers = mouseEvent.getModifiers();
      if (!isAltMask(modifiers) || offset >= selStart && offset < selEnd) {
        disposeHighlighter();
        myTooltipProvider = null;
        return;
      }
      myTooltipProvider = new TooltipProvider(editor, pos);
      myTooltipProvider.execute();
    }
  };

  static {
    ourReferenceAttributes = new TextAttributes();
    ourReferenceAttributes.setForegroundColor(Color.BLUE);
    ourReferenceAttributes.setEffectColor(Color.BLUE);
    ourReferenceAttributes.setEffectType(EffectType.LINE_UNDERSCORE);
  }

  public ArrayIndexer(Project project, StartupManager startupManager) {
    myProject = project;
    startupManager.registerPostStartupActivity(new Runnable() {
      public void run() {
        EditorEventMulticaster eventMulticaster = EditorFactory.getInstance().getEventMulticaster();
        eventMulticaster.addEditorMouseListener(myEditorMouseAdapter);
        eventMulticaster.addEditorMouseMotionListener(myEditorMouseMotionListener);
      }
    });
  }

  @NotNull
  public String getComponentName() {
    return ArrayIndexer.class.getName();
  }

  public void initComponent() {
  }

  public void disposeComponent() {
  }

  public void projectOpened() {
  }

  public void projectClosed() {
    EditorEventMulticaster eventMulticaster = EditorFactory.getInstance().getEventMulticaster();
    eventMulticaster.removeEditorMouseListener(myEditorMouseAdapter);
    eventMulticaster.removeEditorMouseMotionListener(myEditorMouseMotionListener);
  }

  private static boolean isAltMask(int modifiers) {
    int mask = SystemInfo.isMac ? InputEvent.ALT_GRAPH_MASK : InputEvent.ALT_MASK;
    return modifiers == mask;
  }

  private static class InfoGenerator {
    @Nullable
    public static String generateInfo(PsiElement element) {
      StringBuffer sb = new StringBuffer(Bundle.message("array.element.index")).append(" : ");

      PsiField psiField = PsiTreeUtil.getParentOfType(element, PsiField.class);
      if (psiField != null) {
        sb.append(psiField.getName());
      }

      int indexOffset = sb.length();

      PsiArrayInitializerExpression array;
      while ((array = PsiTreeUtil.getParentOfType(element, PsiArrayInitializerExpression.class)) != null) {
        sb.insert(indexOffset, "[" + determineIndex(array, element) + "]");
        element = findPsiExpression(array);
        if (psiField == null) {
          psiField = PsiTreeUtil.getParentOfType(array, PsiField.class);
        }
      }

      return sb.toString();
    }

    private static int determineIndex(PsiArrayInitializerExpression array, PsiElement element) {
      PsiExpression[] initializers = array.getInitializers();

      int index = -1;

      for (int i = 0; i < initializers.length; i++) {
        PsiExpression initializer = initializers[i];
        if (initializer == element) {
          index = i;
          break;
        }
      }

      return index;
    }
  }

  private static class Info {
    public final PsiElement myElementAtPointer;
    public final int myStartOffset;
    public final int myEndOffset;

    public Info(PsiElement elementAtPointer) {
      myElementAtPointer = elementAtPointer;
      myStartOffset = elementAtPointer.getTextOffset();
      myEndOffset = myStartOffset + elementAtPointer.getTextLength();
    }

    public Info(PsiElement elementAtPointer, int startOffset, int endOffset) {
      myElementAtPointer = elementAtPointer;
      myStartOffset = startOffset;
      myEndOffset = endOffset;
    }
  }

  @Nullable
  private Info getInfoAt(final Editor editor, LogicalPosition pos) {
    Document document = editor.getDocument();
    PsiFile file = PsiDocumentManager.getInstance(myProject).getPsiFile(document);
    if (file == null) {
      return null;
    }
    PsiDocumentManager.getInstance(myProject).commitAllDocuments();
    final int offset = editor.logicalPositionToOffset(pos);
    int selStart = editor.getSelectionModel().getSelectionStart();
    int selEnd = editor.getSelectionModel().getSelectionEnd();
    if (offset >= selStart && offset < selEnd) {
      return null;
    }

    PsiElement elementAtPointer = file.findElementAt(offset);

    if (elementAtPointer == null) {
      return null;
    }

    PsiArrayInitializerExpression array = PsiTreeUtil.getParentOfType(elementAtPointer, PsiArrayInitializerExpression.class);

    if (array == null) {
      return null;
    }

    PsiExpression psiExpression = findPsiExpression(elementAtPointer);

    if (psiExpression == null) {
      return null;
    }

    return new Info(psiExpression);
  }

  private void disposeHighlighter() {
    if (myHighlighter != null) {
      myHighlighterView.getMarkupModel().removeHighlighter(myHighlighter);
      Component internalComponent = myHighlighterView.getContentComponent();
      internalComponent.setCursor(myStoredCursor);
      internalComponent.removeKeyListener(myEditorKeyListener);
      myHighlighterView.getScrollingModel().removeVisibleAreaListener(myVisibleAreaListener);
      FileEditorManager.getInstance(myProject).removeFileEditorManagerListener(myFileEditorManagerListener);
      HintManager hintManager = HintManager.getInstance();
      hintManager.hideAllHints();
      myHighlighter = null;
      myHighlighterView = null;
      myStoredCursor = null;
    }
    myStoredInfo = null;
  }

  private class TooltipProvider {
    private final Editor myEditor;
    private final LogicalPosition myPosition;
    private boolean myBrowseType;

    public TooltipProvider(Editor editor, LogicalPosition pos) {
      myEditor = editor;
      myPosition = pos;
    }

    public boolean isTypeBrowsed() {
      return myBrowseType;
    }

    public void execute() {
      myBrowseType = true;
      Info info = getInfoAt(myEditor, myPosition);
      if (info == null) {
        return;
      }
      Component internalComponent = myEditor.getContentComponent();
      if (myHighlighter != null) {
        if (!Comparing.equal(info.myElementAtPointer, myStoredInfo.myElementAtPointer) || info.myStartOffset != myStoredInfo.myStartOffset || info.myEndOffset != myStoredInfo.myEndOffset) {
          disposeHighlighter();
        } else { /* highlighter already set */
          return;
        }
      }
      if (info.myElementAtPointer != null) {
        installLinkHighlighter(info);
        internalComponent.addKeyListener(myEditorKeyListener);
        myEditor.getScrollingModel().addVisibleAreaListener(myVisibleAreaListener);
        myStoredCursor = internalComponent.getCursor();
        myStoredInfo = info;
        FileEditorManager.getInstance(myProject).addFileEditorManagerListener(myFileEditorManagerListener);
        String text = InfoGenerator.generateInfo(info.myElementAtPointer);
        if (text == null) {
          return;
        }
        JLabel label = HintUtil.createInformationLabel(text);
        label.setUI(new MultiLineLabelUI());
        Font FONT = UIUtil.getLabelFont();
        label.setFont(FONT);
        final LightweightHint hint = new LightweightHint(label);
        final HintManager hintManager = HintManager.getInstance();
        label.addMouseMotionListener(new MouseMotionAdapter() {
          public void mouseMoved(MouseEvent e) {
            hintManager.hideAllHints();
          }
        });
        Point p = hintManager.getHintPosition(hint, myEditor, myPosition, HintManager.ABOVE);
        hintManager.showEditorHint(hint, myEditor, p, HintManager.HIDE_BY_ANY_KEY | HintManager.HIDE_BY_TEXT_CHANGE | HintManager.HIDE_BY_SCROLLING, 0, false);
      }
    }

    private void installLinkHighlighter(Info info) {
      int startOffset = info.myStartOffset;
      int endOffset = info.myEndOffset;
      myHighlighter = myEditor.getMarkupModel().addRangeHighlighter(startOffset, endOffset, HighlighterLayer.SELECTION + 1, ourReferenceAttributes, HighlighterTargetArea.EXACT_RANGE);
      myHighlighterView = myEditor;
    }
  }

  private static PsiExpression findPsiExpression(PsiElement element) {
    PsiArrayInitializerExpression array = PsiTreeUtil.getParentOfType(element, PsiArrayInitializerExpression.class);

    if (array == null) {
      return null;
    }

    for (int i = 0; i < array.getInitializers().length; i++) {
      PsiExpression psiExpression = array.getInitializers()[i];
      if (psiExpression == element || PsiTreeUtil.isAncestor(psiExpression, element, true)) {
        return psiExpression;
      }
    }
    return null;
  }
}
