/*
 * Copyright 2006 KonaWorks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.konaworks.idea.guttermenu;

import com.intellij.ide.bookmarks.BookmarkManager;
import com.intellij.ide.bookmarks.EditorBookmark;
import com.intellij.ide.bookmarks.actions.ToggleBookmarkAction;
import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DefaultActionGroup;
import com.intellij.openapi.actionSystem.Separator;
import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.editor.EditorFactory;
import com.intellij.openapi.editor.event.EditorMouseAdapter;
import com.intellij.openapi.editor.event.EditorMouseEvent;
import com.intellij.openapi.editor.event.EditorMouseEventArea;
import com.intellij.openapi.ui.Messages;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

public class GutterMenu implements ProjectComponent {
  private EditorMouseAdapter listener = new EditorMouseAdapter() {
    public void mouseClicked(EditorMouseEvent event) {
      if (!event.getMouseEvent().isPopupTrigger()) {
        return;
      }

      if (!EditorMouseEventArea.LINE_MARKERS_AREA.equals(event.getArea())) {
        return;
      }          

      final Editor editor = event.getEditor();
      final ActionManager actionManager = ActionManager.getInstance();
      final DefaultActionGroup group = new DefaultActionGroup("", true);

      group.add(new ToggleBookmarkAction()); // always enabled, like in the view menu
      group.add(createToggleNumberedBookmarkActionGroup(event.getEditor()));

      final EditorBookmark bookmark = BookmarkManager.getInstance(editor.getProject()).
          findEditorBookmark(editor.getDocument(), editor.getCaretModel().getLogicalPosition().line);

      if (bookmark != null) {
        group.add(new AnAction("Edit description of Bookmark") {
          public void actionPerformed(AnActionEvent e) {
            final String description = Messages.showInputDialog("Please enter a description for the bookmark.", "Bookmark Description", null, bookmark.getDescription(), null);
            bookmark.setDescription(description);
          }
        });
      }

      group.add(Separator.getInstance());

      group.add(actionManager.getAction("ToggleLineBreakpoint"));
      group.add(actionManager.getAction("ToggleMethodBreakpoint"));
      group.add(actionManager.getAction("ToggleFieldBreakpoint"));

      group.add(Separator.getInstance());

      group.add(actionManager.getAction("EditorToggleShowLineNumbers"));

      ActionManager.getInstance().createActionPopupMenu("", group).getComponent().
          show(event.getMouseEvent().getComponent(), event.getMouseEvent().getX(), event.getMouseEvent().getY());
    }
  };

  private DefaultActionGroup createToggleNumberedBookmarkActionGroup(Editor editor) {
    final DefaultActionGroup action = new DefaultActionGroup("Toggle &Numbered Bookmark", true);

    final BookmarkManager bookmarkManager = BookmarkManager.getInstance(editor.getProject());
    final EditorBookmark bookmark = bookmarkManager.findEditorBookmark(editor.getDocument(), editor.getCaretModel().getLogicalPosition().line);
    if (bookmark != null) {
      action.getTemplatePresentation().setEnabled(false);
    } else {
      ActionManager actionManager = ActionManager.getInstance();
      action.add(actionManager.getAction("ToggleBookmark0"));
      action.add(actionManager.getAction("ToggleBookmark1"));
      action.add(actionManager.getAction("ToggleBookmark2"));
      action.add(actionManager.getAction("ToggleBookmark3"));
      action.add(actionManager.getAction("ToggleBookmark4"));
      action.add(actionManager.getAction("ToggleBookmark5"));
      action.add(actionManager.getAction("ToggleBookmark6"));
      action.add(actionManager.getAction("ToggleBookmark7"));
      action.add(actionManager.getAction("ToggleBookmark8"));
      action.add(actionManager.getAction("ToggleBookmark9"));
    }

    return action;
  }

  public void projectOpened() {
    EditorFactory.getInstance().getEventMulticaster().addEditorMouseListener(listener);
  }

  public void projectClosed() {
    EditorFactory.getInstance().getEventMulticaster().removeEditorMouseListener(listener);
  }

  @NonNls
  @NotNull
  public String getComponentName() {
    return GutterMenu.class.getName();
  }

  public void initComponent() {
  }

  public void disposeComponent() {
  }
}
