/*
 * Copyright 2006 KonaWorks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.konaworks.idea.javadocroots;

import com.intellij.ide.BrowserUtil;
import com.intellij.javaee.serverInstances.ApplicationServersManager;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.module.Module;
import com.intellij.openapi.module.ModuleManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.projectRoots.ProjectJdk;
import com.intellij.openapi.roots.ModuleRootManager;
import com.intellij.openapi.roots.OrderRootType;
import com.intellij.openapi.roots.ProjectRootManager;
import com.intellij.openapi.roots.libraries.Library;
import com.intellij.openapi.roots.libraries.LibraryTable;
import com.intellij.openapi.roots.libraries.LibraryTablesRegistrar;
import com.intellij.openapi.ui.popup.JBPopupFactory;
import com.intellij.openapi.ui.popup.ListPopup;
import com.intellij.openapi.ui.popup.PopupStep;
import com.intellij.openapi.ui.popup.util.BaseListPopupStep;
import com.intellij.openapi.util.IconLoader;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.Icon;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

public class ShowJavaDocRootsAction extends AnAction {
  public void actionPerformed(AnActionEvent e) {
    createAndShowPopup((Project) e.getDataContext().getData(DataConstants.PROJECT));
  }

  private void createAndShowPopup(@Nullable final Project project) {
    final List<JavaDocRootsHolder> rootsHolders = new ArrayList<JavaDocRootsHolder>();

    // First get libraries with javadoc from module/project/global & filter those for duplicates regarding the roots.
    // Then module javadoc
    // After that AppServer javaodc and last: project & module jdk javadoc.

    // for filtering a LinkedHashSet is used, so that the order is maintained.
    final Set<JavaDocRootsHolder> librariesJavaDoc = new LinkedHashSet<JavaDocRootsHolder>();
    final LibraryTablesRegistrar libraryTablesRegistrar = LibraryTablesRegistrar.getInstance();
    if (project != null) { // Action is enabled if no project is available...
      // module libs first
      final Module[] modules = ModuleManager.getInstance(project).getModules();
      for (Module module : modules) {
        final LibraryTable libraryTable = ModuleRootManager.getInstance(module).getModifiableModel().getModuleLibraryTable();
        addLibraries(librariesJavaDoc, libraryTable);
      }

      // project libraries
      final LibraryTable projectLibTable = libraryTablesRegistrar.getLibraryTableByLevel(LibraryTablesRegistrar.PROJECT_LEVEL, project);
      addLibraries(librariesJavaDoc, projectLibTable);
    }
    // global/application level libs go last
    final LibraryTable globalLibTable = libraryTablesRegistrar.getLibraryTableByLevel(LibraryTablesRegistrar.APPLICATION_LEVEL, project);
    addLibraries(librariesJavaDoc, globalLibTable);

    rootsHolders.addAll(librariesJavaDoc);

    // AppServer libs (filtered as well, but separate)
    final Set<JavaDocRootsHolder> appSrvLibJavaDoc = new LinkedHashSet<JavaDocRootsHolder>();
    addLibraries(appSrvLibJavaDoc, ApplicationServersManager.getInstance().getLibraryTable());
    rootsHolders.addAll(appSrvLibJavaDoc);

    if (project != null) {
      // Module javadoc
      final Module[] modules = ModuleManager.getInstance(project).getModules();
      for (Module module : modules) {
        if (ModuleRootManager.getInstance(module).getJavadocUrls().length != 0) {
          rootsHolders.add(JavaDocRootsHolderFactory.create(module));
        }
      }
    }
    if (project != null) {
      // JDK's: first get all jdk's (project & module) and filter for duplicates
      final Set<JavaDocRootsHolder> jdksJavaDoc = new LinkedHashSet<JavaDocRootsHolder>();
      final ProjectJdk projectJdk = ProjectRootManager.getInstance(project).getProjectJdk();
      if (projectJdk != null && projectJdk.getRootProvider().getUrls(OrderRootType.JAVADOC).length != 0) {
        jdksJavaDoc.add(JavaDocRootsHolderFactory.create(projectJdk));
      }
      final Module[] modules = ModuleManager.getInstance(project).getModules();
      for (Module module : modules) {
        final ProjectJdk jdk = ModuleRootManager.getInstance(module).getJdk();
        if (jdk != null && jdk.getRootProvider().getUrls(OrderRootType.JAVADOC).length != 0) {
          jdksJavaDoc.add(JavaDocRootsHolderFactory.create(jdk));
        }
      }
      rootsHolders.addAll(jdksJavaDoc);
    }

    if (rootsHolders.isEmpty()) {
      rootsHolders.add(JavaDocRootsHolderFactory.createNoItemsAvailableMarker());
    }
    
    final ListPopup popup = JBPopupFactory.getInstance().createListPopup(new JavaDocRootsPopupStep(rootsHolders));
    popup.showCenteredInCurrentWindow(project); // n.b. this works ok if project == null
  }

  private static void addLibraries(Set<JavaDocRootsHolder> librariesJavaDoc, LibraryTable libraryTable) {
    for (Library library : libraryTable.getLibraries()) {
      final String[] javadocUrls = library.getUrls(OrderRootType.JAVADOC);
      if (javadocUrls != null && javadocUrls.length != 0) {
        librariesJavaDoc.add(JavaDocRootsHolderFactory.create(library));
      }
    }
  }

  private static class JavaDocRootsPopupStep extends BaseListPopupStep<JavaDocRootsHolder> {
    JavaDocRootsPopupStep(List<JavaDocRootsHolder> libraries) {
      super(Bundle.message("title.popup.javadoc.chooser"), libraries);
    }

    @NotNull
    public String getTextFor(JavaDocRootsHolder value) {
      return value.getName();
    }

    public boolean isSpeedSearchEnabled() {
      return true;
    }

    public boolean isSelectable(JavaDocRootsHolder value) {
      return value.hasRoots();
    }

    public Icon getIconFor(JavaDocRootsHolder aValue) {
      return aValue.getIcon();
    }

    public boolean hasSubstep(JavaDocRootsHolder selectedValue) {
      return selectedValue.hasMultipleRoots();
    }

    public PopupStep onChosen(JavaDocRootsHolder selectedValue, boolean finalChoice) {
      // Empty title string results in no title
      if (selectedValue.hasSingleRoot()) {
        openUrl(selectedValue.getRoots()[0]);
        return FINAL_CHOICE;
      } else {
        return new UrlListPopupStep(selectedValue);
      }
    }

    private static void openUrl(String url) {
      BrowserUtil.launchBrowser(url + (!url.endsWith("/") && !url.endsWith("\\") ? "/" : "") + "index.html");
    }

    private static class UrlListPopupStep extends BaseListPopupStep<String> {
      private static final Icon JAVADOC_ICON = IconLoader.getIcon("/modules/javadoc.png");

      public UrlListPopupStep(JavaDocRootsHolder javaDocRootsHolder) {
        super("", Arrays.asList(javaDocRootsHolder.getRoots()), JAVADOC_ICON);
      }

      public PopupStep onChosen(String selectedValue, boolean finalChoice) {
        openUrl(selectedValue);
        return FINAL_CHOICE;
      }
    }
  }
}
