/*
 * Copyright 2006 KonaWorks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.konaworks.idea.toolwindow;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.popup.JBPopup;
import com.intellij.openapi.ui.popup.PopupChooserBuilder;
import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowManager;
import com.intellij.peer.PeerFactory;
import com.intellij.ui.ColoredListCellRenderer;
import com.intellij.ui.SimpleTextAttributes;

import javax.swing.JList;
import javax.swing.ListSelectionModel;
import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class JumpToToolWindowAction extends AnAction {

  private static class MyToolWindowWrapper implements Comparable {
    // This class is needed for the sorting and speedsearch
    // (the latter uses 'toString' on the elements of the list).
    private final ToolWindow toolWindow;
    private final String windowId;

    public MyToolWindowWrapper(ToolWindow toolWindow, String windowId) {
      this.toolWindow = toolWindow;
      this.windowId = windowId;
    }

    public ToolWindow getToolWindow() {
      return toolWindow;
    }

    public String getWindowId() {
      return windowId;
    }

    public String toString() {
      return windowId; // same as the 'fragment' that is appended in the ColoredListCellRenderer.
    }

    public boolean equals(Object o) {
      if (this == o) return true;
      if (o == null || getClass() != o.getClass()) return false;

      final MyToolWindowWrapper that = (MyToolWindowWrapper) o;

      return toolWindow.equals(that.toolWindow);
    }

    public int hashCode() {
      return toolWindow.hashCode();
    }

    public int compareTo(Object o) {
      return windowId.compareTo(((MyToolWindowWrapper) o).getWindowId());
    }
  }

  private static class ToolWindowCellRenderer extends ColoredListCellRenderer {

    protected void customizeCellRenderer(JList list, Object value, int index, boolean selected, boolean hasFocus) {
      if (value instanceof MyToolWindowWrapper) {
        final MyToolWindowWrapper myToolWindowWrapper = (MyToolWindowWrapper) value;
        setIcon(myToolWindowWrapper.getToolWindow().getIcon());
        append(myToolWindowWrapper.getWindowId(), new SimpleTextAttributes(Font.PLAIN, Color.BLACK));
      }
    }
  }

  public void actionPerformed(AnActionEvent e) {
    createAndShowPopup((Project) e.getDataContext().getData(DataConstants.PROJECT));
  }

  public void update(AnActionEvent e) {
    // Enabled when a project is available/open.
    e.getPresentation().setEnabled(e.getDataContext().getData(DataConstants.PROJECT) != null);
  }

  private void createAndShowPopup(final Project project) {
    // Item collection
    final List<MyToolWindowWrapper> toolWindows = new ArrayList<MyToolWindowWrapper>();

    final ToolWindowManager toolWindowManager = ToolWindowManager.getInstance(project);

    for (String windowId : toolWindowManager.getToolWindowIds()) {
      final ToolWindow window = toolWindowManager.getToolWindow(windowId);
      if (window.isAvailable()) {
        toolWindows.add(new MyToolWindowWrapper(window, windowId));
      }
    }

    Collections.sort(toolWindows);

    // List formatting, selection and speedsearch
    final JList jList = new JList(toolWindows.toArray(new MyToolWindowWrapper[toolWindows.size()]));
    jList.setCellRenderer(new ToolWindowCellRenderer());
    jList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    PeerFactory.getInstance().getUIHelper().installListSpeedSearch(jList);

    // Popup preparation, action impl and creation
    final PopupChooserBuilder popupChooserBuilder = new PopupChooserBuilder(jList);
    popupChooserBuilder.setTitle("Tool Windows").setMovable(true);

    popupChooserBuilder.setItemChoosenCallback(new Runnable() {
      public void run() {
        ((MyToolWindowWrapper) jList.getSelectedValue()).getToolWindow().activate(null);
      }
    });

    final JBPopup popup = popupChooserBuilder.createPopup();
    popup.showCenteredInCurrentWindow(project);
  }
}
