package com.konaworks.idea.inspections;

import com.intellij.codeHighlighting.HighlightDisplayLevel;
import com.intellij.codeInsight.daemon.GroupNames;
import com.intellij.codeInspection.InspectionManager;
import com.intellij.codeInspection.LocalInspectionTool;
import com.intellij.codeInspection.LocalQuickFix;
import com.intellij.codeInspection.ProblemDescriptor;
import com.intellij.codeInspection.ProblemHighlightType;
import com.intellij.openapi.project.Project;
import com.intellij.psi.JavaTokenType;
import com.intellij.psi.PsiComment;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import com.intellij.psi.PsiJavaFile;
import com.intellij.psi.PsiWhiteSpace;
import com.intellij.psi.tree.IElementType;
import com.intellij.psi.util.PsiTreeUtil;
import com.intellij.util.IncorrectOperationException;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

public class JavaCvsHistoryInspection extends LocalInspectionTool {

  @NotNull
  public String getShortName() {
    return "KW_JavaCvsHistoryInspection";
  }

  @NotNull
  @Override
  public HighlightDisplayLevel getDefaultLevel() {
    return HighlightDisplayLevel.WARNING;
  }

  public boolean isEnabledByDefault() {
    return true;
  }

  @NotNull
  public String getGroupDisplayName() {
    return GroupNames.MATURITY_GROUP_NAME;
  }

  @NotNull
  public String getDisplayName() {
    return Bundle.message("inspection.name.KW_JavaCvsHistoryInspection");
  }


  public ProblemDescriptor[] checkFile(@NotNull final PsiFile file, @NotNull final InspectionManager manager, boolean flag) {
    final List<ProblemDescriptor> problems = new ArrayList<ProblemDescriptor>();

    if (file instanceof PsiJavaFile) {
      PsiElement firstComment = null;
      for (PsiElement element : file.getChildren()) {
        if (isEndOfLineComment(element)) {
          firstComment = element;
          break;
        }
      }
      boolean containsCvsLog = false;
      if (firstComment != null) {
        PsiElement nextComment = firstComment;
        PsiElement lastComment;

        while (true) {
          if (nextComment != null && nextComment.getText().contains("$Log")) {
            containsCvsLog = true;
          }
          lastComment = nextComment;
          nextComment = PsiTreeUtil.skipSiblingsForward(nextComment, PsiWhiteSpace.class);
          if (!isEndOfLineComment(nextComment)) {
            break;
          }
        }
        if (containsCvsLog) {
          problems.add(manager.createProblemDescriptor(firstComment, lastComment, "cvs log", ProblemHighlightType.GENERIC_ERROR_OR_WARNING, REMOVE_QUICK_FIX));
        }
      }
    }

    return problems.toArray(new ProblemDescriptor[problems.size()]);
  }

  private LocalQuickFix REMOVE_QUICK_FIX = new RemoveQuickFix();

  private static class RemoveQuickFix implements LocalQuickFix {
    @NotNull
    public String getName() {
      return Bundle.message("quickfix.remove.cvs.log");
    }

    public void applyFix(@NotNull Project project, ProblemDescriptor descriptor) {
      try {
        descriptor.getStartElement().getParent().deleteChildRange(descriptor.getStartElement(), descriptor.getEndElement());
      }
      catch (IncorrectOperationException e) {
        // do nothing
      }
    }

    @NotNull
    public String getFamilyName() {
      return getName();
    }
  }

  private static boolean isEndOfLineComment(PsiElement element) {
    if (!(element instanceof PsiComment)) {
      return false;
    }
    final PsiComment comment = (PsiComment) element;
    final IElementType tokenType = comment.getTokenType();
    return JavaTokenType.END_OF_LINE_COMMENT.equals(tokenType);
  }
}
