/*
 * Copyright 2006 KonaWorks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.konaworks.idea.modulegrouping;

import com.intellij.openapi.module.ModifiableModuleModel;
import com.intellij.openapi.module.Module;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.DialogBuilder;
import org.apache.commons.collections.MultiHashMap;
import org.jetbrains.annotations.Nullable;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

public abstract class AbstractGroupByDistributor implements ModulesToGroupsDistributor {

  private JPanel combinationPanel;
  private JCheckBox createGroupsForSingleModuleCheckBox;
  private JPanel specificOptionsPanel;

  // todo (usability enhancement) : store settings used.? don't know: action is used infrequently
  private boolean createGroupsForSingleModule = false;

  final public void distribute(Module[] modules, ModifiableModuleModel modifiableModuleModel) {
    if (modules == null || modules.length == 0) {
      return;
    }

    // Group da modules using the subclass implementations.
    // A hashmap is used to support the option 'createGroupsForSingleModule'.
    final MultiHashMap moduleBaskets = new MultiHashMap();
    for (Module module : modules) {
      moduleBaskets.put(getGroupName(module), module);
    }

    // filter out too small groups
    if (!createGroupsForSingleModule) {
      final Set keySet = moduleBaskets.keySet();
      final Object[] basketKeys = keySet.toArray();
      for (Object basketKey : basketKeys) {
        if (moduleBaskets.getCollection(basketKey).size() == 1) {
          // move that module to top level
          final List removedModules = (ArrayList) moduleBaskets.remove(basketKey);
          moduleBaskets.put(null, removedModules.get(0));
        }
      }
    }

    // apply to moduleModel
    final Set basketKeys = moduleBaskets.keySet();
    for (Object basketKey : basketKeys) {
      final Collection collection = moduleBaskets.getCollection(basketKey);
      for (Object obj : collection) {
        if (basketKey != null) {
          modifiableModuleModel.setModuleGroupPath((Module) obj, new String[]{(String) basketKey});
        } else {
          modifiableModuleModel.setModuleGroupPath((Module) obj, null);
        }
      }
    }
  }

  public final boolean showOptionsPopup(Project project) {
    final JComponent subclassOptionsPanel = createOptionsPanel();
    if (subclassOptionsPanel == null) {
      this.specificOptionsPanel.setVisible(false);
    } else {
      this.specificOptionsPanel.add(subclassOptionsPanel, BorderLayout.CENTER);
    }

    final DialogBuilder builder = new DialogBuilder(project);
    builder.setCenterPanel(combinationPanel);
    builder.setTitle(Bundle.message("group.modules.options.title"));
    builder.getDialogWrapper().pack();
    builder.show();
    if (builder.getDialogWrapper().isOK()) {
      applyMyOptions();
      applyOptions();
      return true;
    } else {
      return false;
    }
  }

  private void applyMyOptions() {
    createGroupsForSingleModule = createGroupsForSingleModuleCheckBox.isSelected();
  }

  /**
   * Determine the group path for the module.
   * @param module the module that needs to be (re)grouped.
   * @return the path of the group as an array of group names starting from the project root.
   */
  protected abstract String getGroupName(Module module);

  /**
   * Template method which can be overriden to provide type specific options
   * @return options component
   */
  @Nullable
  JPanel createOptionsPanel() {
    return null;
  }

  /**
   * Template method to apply the options from the options panel to any 'memory objects'.
   */
  void applyOptions() {

  }
}
