/*
 * Copyright 2006 KonaWorks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.konaworks.idea.modulegrouping;

import com.intellij.openapi.module.Module;
import org.jetbrains.annotations.Nullable;

import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

public class GroupByNamePartDistributor extends AbstractGroupByDistributor {
  private PartType partType;
  private int numberOfCamelHumps;
  private JRadioButton byPrefixRadioButton;
  private JRadioButton bySuffixRadioButton;
  private JTextField numberOfCamelHumpsTextField;
  private JPanel optionsPanel;

  @SuppressWarnings({"SameParameterValue"})
  public GroupByNamePartDistributor(PartType partType) {
    this.partType = partType;
    switch (partType) {
      case SUFFIX:
        bySuffixRadioButton.setSelected(true);
        break;
      case PREFIX:
      default:
        byPrefixRadioButton.setSelected(true);
    }
    numberOfCamelHumpsTextField.setText("1");
  }

  protected String getGroupName(Module module) {
    if (PartType.SUFFIX.equals(partType)) {
      return getSuffixWithLength(module.getName(), numberOfCamelHumps);
    } else {
      return getPrefixWithLength(module.getName(), numberOfCamelHumps);
    }
  }

  public enum PartType {
    PREFIX,
    SUFFIX
  }

  @Nullable
  protected JPanel createOptionsPanel() {
    return optionsPanel;
  }

  protected void applyOptions() {
    if (bySuffixRadioButton.isSelected()) {
      partType = PartType.SUFFIX;
    } else {
      partType = PartType.PREFIX;
    }
    try {
      numberOfCamelHumps = Integer.parseInt(numberOfCamelHumpsTextField.getText());
    } catch (NumberFormatException e) {
      numberOfCamelHumps = 1;
    }
  }

  static String getPrefixWithLength(String name, int numberOfWordsToBeFound) {
    // The list can be endless, but these are most commonly used :
    final String wordSplitters = new String(new char[]{'.', ',', '_', '-', '~', '@', '#', '$', '%', '^', '&', '(', ')', '+', '='});

    int newWordStart = 0;
    final char[] chars = name.toCharArray();
    int numberOfWordsFound = 0;
    for (int currentWordNumber = 0; currentWordNumber < numberOfWordsToBeFound; currentWordNumber++) {
      boolean wordStartFound = false;
      for (int i = newWordStart; !wordStartFound && i < chars.length - 1; i++) {

        if (Character.isLowerCase(chars[i]) && Character.isUpperCase(chars[i + 1])
            || Character.isLowerCase(chars[i]) && Character.isWhitespace(chars[i+1])
            || wordSplitters.indexOf(chars[i]) == -1 && wordSplitters.indexOf(chars[i + 1]) != -1
            || Character.isUpperCase(chars[i]) && Character.isUpperCase(chars[i+1]) && i + 2 < chars.length && Character.isLowerCase(chars[i+2])
            ) {
          newWordStart = i + 1;
          wordStartFound = true;
          numberOfWordsFound++;
        }
      }
    }
    if (numberOfWordsFound == numberOfWordsToBeFound) {
      return name.substring(0, newWordStart);
    } else {
      return name.substring(0);
    }
  }

  static String getSuffixWithLength(String name, int numberOfWordsToBeFound) {
    // The list can be endless, but these are most commonly used :
    final String wordSplitters = new String(new char[]{'.', ',', '_', '-', '~', '@', '#', '$', '%', '^', '&', '(', ')', '+', '='});

    int newWordEnd = name.length() - 1;
    final char[] chars = name.toCharArray();
    int numberOfWordsFound = 0;
    for (int currentWordNumber = 0; currentWordNumber < numberOfWordsToBeFound; currentWordNumber++) {
      boolean wordEndFound = false;
      for (int i = newWordEnd; !wordEndFound && i > 1; i--) {

        if (Character.isUpperCase(chars[i]) && Character.isLowerCase(chars[i - 1])
            || Character.isUpperCase(chars[i]) && Character.isWhitespace(chars[i-1])
            || wordSplitters.indexOf(chars[i]) == -1 && wordSplitters.indexOf(chars[i-1]) != -1
            ) {
          newWordEnd = i - 1;
          wordEndFound = true;
          numberOfWordsFound++;
        } else if (Character.isLowerCase(chars[i]) && Character.isUpperCase(chars[i - 1]) && i - 2 > 0 && Character.isUpperCase(chars[i - 2])
            ) {
          newWordEnd = i - 2;
          wordEndFound = true;
          numberOfWordsFound++;
        }
      }
    }
    if (numberOfWordsFound == numberOfWordsToBeFound) {
      return name.substring(newWordEnd + 1);
    } else {
      return name.substring(0);
    }
  }
}
