/*
 * Copyright 2006 KonaWorks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.konaworks.idea.modulegrouping;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.actionSystem.Presentation;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.module.ModifiableModuleModel;
import com.intellij.openapi.module.Module;
import com.intellij.openapi.module.ModuleManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.roots.ModuleCircularDependencyException;

@SuppressWarnings({"ComponentNotRegistered"})
class GroupModulesAction extends AnAction {
  private final GroupingType groupingType;

  public GroupModulesAction(GroupingType groupingType) {
    this.groupingType = groupingType;
  }

  public void update(AnActionEvent event) {
    final Presentation presentation = event.getPresentation();
    presentation.setText(getText());
    presentation.setDescription(getDescription());
  }

  private String getText() {
    switch (groupingType) {
      case BY_TYPE:
        return Bundle.message("groupingType.byType.menuText");
      case BY_NAME_PREFIX_OR_SUFFIX:
        return Bundle.message("groupingType.byPrefixSuffix.menuText");
      case BY_PARENT_DIR:
        return Bundle.message("groupingType.byParentDir.menuText");
       case BY_DEV_KIT:
         return Bundle.message("groupingType.byDevKit.menuText");
       case BY_LANGUAGE_LEVEL:
         return Bundle.message("groupingType.byLanguageLevel.menuText");
       case BY_VCS_TYPE:
         return Bundle.message("groupingType.byVCSType.menuText");
      default:
        throw new IllegalStateException("groupingType not supported.");
    }
  }

  private String getDescription() {
    switch (groupingType) {
      case BY_TYPE:
        return Bundle.message("groupingType.byType.description");
      case BY_NAME_PREFIX_OR_SUFFIX:
        return Bundle.message("groupingType.byPrefixSuffix.description");
      case BY_PARENT_DIR:
        return Bundle.message("groupingType.byParentDir.description");
      case BY_DEV_KIT:
        return Bundle.message("groupingType.byDevKit.description");
      case BY_LANGUAGE_LEVEL:
        return Bundle.message("groupingType.byLanguageLevel.description");
      case BY_VCS_TYPE:
        return Bundle.message("groupingType.byVCSType.description");
      default:
        return null;
    }
  }

  public void actionPerformed(AnActionEvent e) {
    final Project project = (Project) e.getDataContext().getData(DataConstants.PROJECT);
    groupTheModules(project, (Module[]) e.getDataContext().getData(DataConstants.MODULE_CONTEXT_ARRAY));
  }

  private void groupTheModules(final Project project, final Module[] selectedModules) {
    final Module[] inputModulesForAction = ModuleSelectionUtil.extendWithImplicitSelection(selectedModules, project);
    if (inputModulesForAction.length == 0) {
      return;
    }

    final ModulesToGroupsDistributor distributor = DistributorFactory.create(groupingType);
    final boolean proceed = distributor.showOptionsPopup(inputModulesForAction[0].getProject());
    if (!proceed) {
      return;
    }

    ApplicationManager.getApplication().runWriteAction(new Runnable() {
      public void run() {
        final ModifiableModuleModel modifiableModuleModel = ModuleManager.getInstance(project).getModifiableModel();
        distributor.distribute(inputModulesForAction, modifiableModuleModel);

        try {
          modifiableModuleModel.commit();
        } catch (ModuleCircularDependencyException e1) {
          // This should not break the action
        }
      }
    });
  }
}
