/*
 * Copyright 2006 KonaWorks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.konaworks.idea.editors;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.CommonShortcuts;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.actionSystem.KeyboardShortcut;
import com.intellij.openapi.actionSystem.Shortcut;
import com.intellij.openapi.editor.markup.TextAttributes;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.fileEditor.OpenFileDescriptor;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.popup.PopupChooserBuilder;
import com.intellij.openapi.util.Iconable;
import com.intellij.openapi.vcs.FileStatus;
import com.intellij.openapi.vcs.FileStatusManager;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.peer.PeerFactory;
import com.intellij.ui.ColoredListCellRenderer;
import com.intellij.ui.SimpleTextAttributes;
import com.intellij.util.IconUtil;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.ListSelectionModel;
import java.awt.Font;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

public class ShowOpenEditorsAction extends AnAction {
  private static class MyVirtualFileWrapper {
    // class is needed for the speedsearch which uses 'toString' on the elements of the list.
    private final VirtualFile virtualFile;

    public MyVirtualFileWrapper(VirtualFile virtualFile) {
      this.virtualFile = virtualFile;
    }

    public VirtualFile getVirtualFile() {
      return virtualFile;
    }

    public String toString() {
      return virtualFile.getName(); // same as the 'fragment' that is appended in the cellRenderer.
    }

    public boolean equals(Object o) {
      if (this == o) return true;
      if (o == null || getClass() != o.getClass()) return false;

      final MyVirtualFileWrapper that = (MyVirtualFileWrapper) o;

      return virtualFile.equals(that.virtualFile);
    }

    public int hashCode() {
      return virtualFile.hashCode();
    }
  }

  private static class EditorCellRenderer extends ColoredListCellRenderer {
    private final Project project;
    private final VirtualFile selectedFile;

    public EditorCellRenderer(Project project, VirtualFile selectedFile) {
      this.project = project;
      this.selectedFile = selectedFile;
    }

    protected void customizeCellRenderer(JList list, Object value, int index, boolean selected, boolean hasFocus) {
      if (value instanceof MyVirtualFileWrapper) {
        final VirtualFile virtualFile = ((MyVirtualFileWrapper) value).getVirtualFile();

        // Don't simply use the virtualFile.getIcon(), since we'd like the 'read-only lock' to be shown.
        setIcon(IconUtil.getIcon(virtualFile, Iconable.ICON_FLAG_READ_STATUS, project));

        final FileStatus fileStatus = FileStatusManager.getInstance(project).getStatus(virtualFile);
        final int fontType = virtualFile.equals(selectedFile) ? Font.BOLD : Font.PLAIN;
        final TextAttributes textattributes = new TextAttributes(fileStatus.getColor(), null, null, null, fontType);

        append(virtualFile.getName(), SimpleTextAttributes.fromTextAttributes(textattributes));
      }
    }
  }

  public void actionPerformed(AnActionEvent e) {
    createAndShowPopup((Project) e.getDataContext().getData(DataConstants.PROJECT));
  }

  public void update(AnActionEvent e) {
    // Enabled when a project is available/open && at least one editor open
    final Project project = (Project) e.getDataContext().getData(DataConstants.PROJECT);

    if (project != null) {
      final VirtualFile[] openFiles = FileEditorManager.getInstance(project).getOpenFiles();
      e.getPresentation().setEnabled(openFiles != null && openFiles.length != 0);
    } else {
      e.getPresentation().setEnabled(false);
    }
  }

  private void createAndShowPopup(final Project project) {
    // Item collection
    final DefaultListModel listModel = new DefaultListModel();
    final FileEditorManager fileEditorManager = FileEditorManager.getInstance(project);

    final VirtualFile[] openFiles = fileEditorManager.getOpenFiles();
    for (VirtualFile openFile : openFiles) {
      listModel.addElement(new MyVirtualFileWrapper(openFile));
    }

    // List with formatting, selection and speedsearch
    final JList list = new JList(listModel);

    final VirtualFile[] selectedFiles = fileEditorManager.getSelectedFiles();
    final VirtualFile selectedFile = selectedFiles.length != 0 ? selectedFiles[0] : null;
    list.setCellRenderer(new EditorCellRenderer(project, selectedFile));

    list.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
    if (selectedFiles.length != 0) {
      list.setSelectedValue(new MyVirtualFileWrapper(selectedFile), true);
    } else if (listModel.getSize() == 0) {
      list.clearSelection();
    }

    PeerFactory.getInstance().getUIHelper().installListSpeedSearch(list);

    addKeyListenerForEditorClose(list, project);

    // Popup creation prep
    final PopupChooserBuilder popupChooserBuilder = new PopupChooserBuilder(list);
    popupChooserBuilder.setTitle(Bundle.message("title.popup.open.editors")).setMovable(true);

    popupChooserBuilder.setItemChoosenCallback(new Runnable() {
      public void run() {
        if (list.getSelectedValue() != null) {
          // Move focus to the most recent editor for the selected file.
          final VirtualFile fileToActivate = ((MyVirtualFileWrapper) list.getSelectedValue()).getVirtualFile();
          final OpenFileDescriptor descriptor = new OpenFileDescriptor(project, fileToActivate);
          fileEditorManager.openEditor(descriptor, true);
        }
      }
    });

    // Enable the 'Jump to Source'/EditSource KeyStrokes (like the default 'F4').
    final Shortcut[] shortcuts = CommonShortcuts.getEditSource().getShortcuts();
    for (Shortcut shortcut : shortcuts) {
      if (shortcut instanceof KeyboardShortcut)
        popupChooserBuilder.addAdditionalChooseKeystroke(((KeyboardShortcut) shortcut).getFirstKeyStroke());
    }

    popupChooserBuilder.createPopup().showCenteredInCurrentWindow(project);
  }

  private void addKeyListenerForEditorClose(final JList list, final Project project) {
    list.addKeyListener(new KeyAdapter() {

      public void keyPressed(KeyEvent keyevent) {
        if (keyevent.getKeyCode() != KeyEvent.VK_DELETE) {
          return;
        }

        final int selectedIndex = list.getSelectedIndex();
        if (selectedIndex == -1 || selectedIndex >= list.getModel().getSize()) {
          return;
        }

        closeEditor(list, project);
      }
    });
  }

  private void closeEditor(JList list, Project project) {
    final DefaultListModel listModel = (DefaultListModel) list.getModel();
    final Object[] selectedValues = list.getSelectedValues();
    for (Object selectedValue : selectedValues) {

      // First remove the element from the list and update selection
      listModel.removeElement(selectedValue);
      final int selectedIndex = list.getSelectedIndex();
      if (listModel.getSize() == 0) {
        list.clearSelection();
      } else {
        if (listModel.getSize() > selectedIndex) {
          // set the selection to the first element after the 'closed file element'
          list.setSelectedIndex(selectedIndex);
        } else { // last element was selected
          list.setSelectedIndex(listModel.getSize() - 1);
        }
      }
      // Perform the actual close action
      final FileEditorManager editorManager = FileEditorManager.getInstance(project);
      final VirtualFile fileToClose = ((MyVirtualFileWrapper) selectedValue).getVirtualFile();
      editorManager.closeFile(fileToClose);

      // todo (fixme) focus is lost on this action
    }
  }
}
